import 'dart:io';
import 'dart:typed_data';
import 'dart:ui' as ui;

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:path_provider/path_provider.dart';

/// 手写签名
/// GlobalKey<CustomSignatureState> keySignature = GlobalKey();
/// keySignature.currentState.clear()

class CustomSignature extends StatefulWidget {
  final Color color;
  final double strokeWidth;

  CustomSignature({Key? key, this.color = Colors.black87, this.strokeWidth = 2}) : super(key: key);

  List<Offset?> _points = <Offset>[];

  @override
  CustomSignatureState createState() => CustomSignatureState();
}

class CustomSignatureState extends State<CustomSignature> {
  GlobalKey _globalKeyRepaintBoundary = GlobalKey();

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onScaleUpdate: (e) {
        if (e.scale == 1) setState(() => widget._points.add(e.localFocalPoint));
      },
      onScaleEnd: (e) => setState(() => widget._points.add(null)),
      behavior: HitTestBehavior.translucent,
      child: RepaintBoundary(
          key: _globalKeyRepaintBoundary, child: Container(color: Colors.white, width: double.infinity, height: double.infinity, child: CustomPaint(painter: _SignatureCustomPainter(widget.color, widget.strokeWidth, widget._points)))),
    );
  }

  void clear() => setState(() => widget._points.clear());

  Future<String> save({String? name}) async {
    RenderRepaintBoundary? boundary = _globalKeyRepaintBoundary.currentContext?.findRenderObject() as RenderRepaintBoundary?;
    ui.Image? image = await boundary?.toImage();
    ByteData? byteData = await image?.toByteData(format: ui.ImageByteFormat.png);
    Uint8List? pngBytes = byteData?.buffer.asUint8List();
    String rootPath = (await getTemporaryDirectory()).path;
    var fileName = name ?? DateTime.now().millisecondsSinceEpoch.toString();
    var path = rootPath + "/" + fileName + ".png";
    File file = File(path);
    if (file.existsSync()) {
      file.deleteSync();
    }
    if (pngBytes != null) {
      file.writeAsBytes(pngBytes);
    }
    return path;
  }
}

class _SignatureCustomPainter extends CustomPainter {
  final Color _color;
  final double _strokeWidth;
  final List<Offset?> _points;
  late Paint _paint;

  _SignatureCustomPainter(this._color, this._strokeWidth, this._points) {
    _paint = Paint()
      ..color = _color
      ..strokeCap = StrokeCap.round
      ..isAntiAlias = true
      ..strokeWidth = _strokeWidth;
  }

  @override
  void paint(ui.Canvas canvas, ui.Size size) {
    for (int i = 1; i < _points.length; i++) {
        Offset? offset = _points[i];
        Offset? offset_1 = _points[i-1];
      if (offset!= null && offset_1 != null) {
        canvas.drawLine(offset_1, offset, _paint);
      }
    }
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return this != oldDelegate;
  }
}
